"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.testDB = exports.dbClient = void 0;
const dynamodb_1 = require("aws-sdk/clients/dynamodb");
const moment = require("moment");
const log_1 = require("./log");
const response_1 = require("./response");
exports.dbClient = (docDB, tableName) => {
    const queryDefaults = {
        TableName: tableName
    };
    return {
        get: async (query) => docDB
            .get({ ...queryDefaults, ...query })
            .promise()
            .then(result => result.Item),
        put: async (query, isNew) => {
            const timestamp = moment().unix();
            const data = (await docDB
                .put({
                ...queryDefaults,
                ...query,
                ConditionExpression: isNew && !query.ConditionExpression ? 'attribute_not_exists(pk) AND attribute_not_exists(sk)' : undefined,
                Item: {
                    ...query.Item,
                    updatedAt: timestamp,
                    createdAt: isNew ? timestamp : query.Item.createdAt
                }
            })
                .promise()
                .then(result => result.Attributes)
                .catch(error => {
                log_1.out.info({ error });
                if (error.code === 'ConditionalCheckFailedException' && isNew) {
                    throw response_1.response(response_1.BAD_REQUEST_400('Resource already exists'));
                }
                throw error;
            }));
            return data;
        },
        update: async (query) => {
            const timestamp = moment().unix();
            return (await docDB
                .update({
                ...queryDefaults,
                ...query,
                UpdateExpression: query.UpdateExpression + ', updatedAt = :updatedAt',
                ExpressionAttributeValues: {
                    ...query.ExpressionAttributeValues,
                    ':updatedAt': timestamp
                }
            })
                .promise()
                .then(result => result.Attributes));
        },
        query: async (query) => {
            log_1.out.info({ query });
            return (await docDB.query({ ...queryDefaults, ...query }).promise());
        },
        scan: async (query) => docDB.scan({ ...queryDefaults, ...query }).promise(),
        delete: async (query) => docDB
            .delete({
            ...queryDefaults,
            ...query,
            ConditionExpression: query.ConditionExpression || 'attribute_exists(pk) AND attribute_exists(sk)'
        })
            .promise()
    };
};
exports.testDB = exports.dbClient(new dynamodb_1.DocumentClient({
    region: 'localhost',
    endpoint: 'http://localhost:8000'
}), 'x-lambda-helpers-db-local');
